###Stage structured script####
#######LIB#######
library(PSPManalysis)
library(ggplot2)
#####SETTINGS######
pars.names=c("Delta","Xmax1","Xmax2","Sigma","Q","T","Amax","Amin","H","Sig","Sb","Sj","Sm","Mur","Xmin",
             "Psi","Meta","Qs","Rho" ,"S")
output = c("In1_L", "In2_L", "Lm", "Lm2","Jm","Am","L","L2","J","A")
Env=c("X1","X2","birthrate")
meta_index=which(pars.names == 'Meta')-1
Sj_index=which(pars.names == 'Sj')-1
psi_index <- which(pars.names == 'Psi')-1
#####Parameters#####
Xmax1=4
Xmax2=2
Qs=0.3  ###The higher, the more expensive (so Qs is actually 1-qs)
Rho=0.5 #Prob to survive metamorphosis
Sm=0.1
Sb= Sm*0.01
Sj=5E-6
Sj_min=5E-6
xmin=0.01
MaxSj=Sm-xmin #Max size where metamorphosis can take place
Psi=5E-6 #Minimum value, otherwise numerical error
Psi_min=5E-6
Meta=0
Mu=0.02
Sig=30
Q=1
Amax=0.6
Amin=0.06
#######Weak trade-off######
S=0.5
Xmax2=0
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
BPfind<-PSPMequi(modelname="Stagestructured", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=Pars,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-Pars[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step 3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Stagestructured",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=Pars,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[16]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.2
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Sj_init=Sj_min
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
canonical_figD <- PSPMevodyn(modelname="Stagestructured", 
                             startpoint=c(startpoint, Psi,meta_init,Sj_init), 
                             curvepars=c(0.0000001,10000000),
                             evopars=c(0,psi_index,Psi_min,1, 0, meta_index,0,1,0,Sj_index,Sj_min,MaxSj),
                             covars = NULL,
                             parameters = Pars,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:17]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
Psi = max(Psi_min,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
Meta=canonicalmeta_figD$META[nrow(canonicalmeta_figD)]
Sj = max(Sj_min,canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)])
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Stagestructured", 
                              startpoint=c(startpnt, Psi,Meta,Sj), 
                              curvepars=c(0.0001,1000000),
                              evopars=c(0,psi_index,Psi_min,1, Psi, meta_index,0,1,0,Sj_index,Sj_min,20),
                              covars = NULL,
                              parameters = Pars,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:17]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
Psi = max(Psi_min,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
Meta=canonicalmeta_figDB$META[nrow(canonicalmeta_figDB)]
Sj = max(Sj_min,canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)])
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)

MetaESS1 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta,Psi ),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1, 0, psi_index,Psi_min,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)
MetaESS2 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta,Psi ),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1, 0, psi_index,Psi_min,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)
colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:16]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
colnames(MetaESS2$curvepoints)[2:4]<-Env 
colnames(MetaESS2$curvepoints)[7:16]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS1_data$ESS=ifelse(MetaESS1_data$`eig J`>0, "ERP", ifelse(MetaESS1_data$`eig H`<0&MetaESS1_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS1_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS1_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 
MetaESS2_data$ESS=ifelse(MetaESS2_data$`eig J`>0, "ERP", ifelse(MetaESS2_data$`eig H`<0&MetaESS2_data$`eig (J+J')/2`<0, 'CSS_strong',ifelse(MetaESS2_data$`eig H`<0, 'CSS_weak',ifelse(MetaESS2_data$`Z^T C01 Z`<=0, 'EBP', "?")))) 

ggplot(data=MetaESS1_data,aes(x=R2max,y=META))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))
#Check with canonical that ERP is not an ERP but a CSS




#Check what happens after the limit point#
Xmax2=1.98160357E+00-0.01
pnt = c(4.68931537E-01, 9.61458325E-01, 1.81339900E+01)
Meta=9.56362811E-01+0.01
Psi=4.36112902E-02

Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
BPfind<-PSPMequi(modelname="Stagestructured", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=Pars,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-Pars[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
NontrivData=as.data.frame(Nontriv$curvepoints)
startpnt=as.numeric(NontrivData[nrow(NontrivData), c(2:4)])
canonical_Lp <- PSPMevodyn(modelname="Stagestructured", 
                              startpoint=c(startpnt, Psi,Meta,Sj), 
                              curvepars=c(0.00001,1000000),
                              evopars=c(0,psi_index,Psi_min,1, Psi, meta_index,0,1,0,Sj_index,Sj_min,MaxSj),
                              covars = NULL,
                              parameters = Pars,
                              options=NULL,
                              clean=TRUE)

#Meta disappears
#Continue ERP of Meta, For simplicity with SJ and Psi in minimum vlaue
#This line is for low values of meta correct. For high values check with use of canonical equation if 
#Meta evolves or not
Xmax2=5.11663
Env=c(0.535925, 4.63322, 5.39299)
Meta=0.000757035
Sj=5E-06 
Psi=0
startpoint = c(0.5364282,
               4.697348,
               5.663510)
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
MetaESS4 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpoint,Meta),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)


colnames(MetaESS4$curvepoints)[2:4]<-Env
colnames(MetaESS4$curvepoints)[7:16]<-output
MetaESS4_data=as.data.frame(MetaESS4$curvepoints)
MetaESS4_data$R0_xx_yy = MetaESS4_data$`R0_xx[16]` - MetaESS4_data$`R0_yy[16]`
MetaESS4_data$ESS = ifelse(is.na(MetaESS4_data$R0_xx_yy), 'ESS', ifelse(MetaESS4_data$R0_xx_yy<0, "ERP", ifelse(MetaESS4_data$`R0_yy[16]`<0, "CSS", "EBP")))
#EBP part not really EBP, since as soon as psi and sj evolve as well, no ebp anymore
MetaESS4_data=MetaESS4_data[c(1:300),]

ggplot(data=MetaESS1_data,aes(x=R2max,y=META))+geom_path(aes(colour=ESS))+
  geom_point(data=MetaESS2_data,aes(colour=ESS))+
  geom_path(data=MetaESS4_data,aes(colour=ESS))

#######Linear trade-off######
S=1
Xmax2=0
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
BPfind<-PSPMequi(modelname="Stagestructured", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=Pars,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-Pars[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step 3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Stagestructured",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=0.1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=Pars,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=subset(NoMetaData,`R0_x[16]`<=0)
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Sj_init=Sj_min
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
canonical_figD <- PSPMevodyn(modelname="Stagestructured", 
                             startpoint=c(startpoint, Psi,meta_init,Sj_init), 
                             curvepars=c(0.0000001,10000000),
                             evopars=c(0,psi_index,Psi_min,1, 0, meta_index,0,1,0,Sj_index,Sj_min,MaxSj),
                             covars = NULL,
                             parameters = Pars,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:17]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
Psi = max(Psi_min,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
Meta=canonicalmeta_figD$META[nrow(canonicalmeta_figD)]
Sj = max(Sj_min,canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)])
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Stagestructured", 
                              startpoint=c(startpnt, Psi,Meta,Sj), 
                              curvepars=c(0.00001,1000000),
                              evopars=c(0,psi_index,Psi_min,1, Psi, meta_index,0,1,0,Sj_index,Sj_min,20),
                              covars = NULL,
                              parameters = Pars,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:17]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
Psi = max(Psi_min,canonicalmeta_figDB$PSI[nrow(canonicalmeta_figDB)])
Meta=canonicalmeta_figDB$META[nrow(canonicalmeta_figDB)]
Sj = max(Sj_min,canonicalmeta_figDB$Xj[nrow(canonicalmeta_figDB)])
startpnt=as.numeric(canonicalmeta_figDB[nrow(canonicalmeta_figDB), c(2:4)])
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)

MetaESS1 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)

MetaESS2 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)

colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:16]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
MetaESS1_data$R0_xx_yy = MetaESS1_data$`R0_xx[16]` - MetaESS1_data$`R0_yy[16]`
MetaESS1_data$ESS = ifelse(is.na(MetaESS1_data$R0_xx_yy), 'ESS', ifelse(MetaESS1_data$R0_xx_yy<0, "ERP", ifelse(MetaESS1_data$`R0_yy[16]`<0, "CSS", "EBP")))

colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:16]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS2_data$R0_xx_yy = MetaESS2_data$`R0_xx[16]` - MetaESS2_data$`R0_yy[16]`
MetaESS2_data$ESS = ifelse(is.na(MetaESS2_data$R0_xx_yy), 'ESS', ifelse(MetaESS2_data$R0_xx_yy<0, "ERP", ifelse(MetaESS2_data$`R0_yy[16]`<0, "CSS", "EBP")))
MetaESS2_data = subset(MetaESS2_data, `R0_x[16]`<=0)

ggplot(data=MetaESS1_data,aes(x=R2max,y=META))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))

##Contintue with Meta = 1
Meta = 1
startpnt=as.numeric(MetaESS2_data[nrow(MetaESS2_data), c(1:4)])
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
MetaESS3<- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(startpnt),
                    stepsize=-0.1, 
                    parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30), 
                    parameters=Pars,
                    options=c("popEVO", "0", 'parEVO',as.character(meta_index),
                              "popEVO", "0", 'parEVO',as.character(psi_index),
                              "popEVO", "0", 'parEVO',as.character(Sj_index)),
                    clean=TRUE)
#Check what happens after the limit point#
lp = MetaESS3$bifpoints
Xmax2=lp[1]-0.01

Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
BPfind<-PSPMequi(modelname="Stagestructured", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=Pars,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-Pars[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
NontrivData=as.data.frame(Nontriv$curvepoints)
startpnt=as.numeric(NontrivData[nrow(NontrivData), c(2:4)])
canonical_Lp <- PSPMevodyn(modelname="Stagestructured", 
                           startpoint=c(startpnt, Psi,Meta,Sj), 
                           curvepars=c(0.00001,1000000),
                           evopars=c(0,psi_index,Psi_min,1, Psi, meta_index,0,1,0,Sj_index,Sj_min,MaxSj),
                           covars = NULL,
                           parameters = Pars,
                           options=NULL,
                           clean=TRUE)

#Meta stays 1, but size at meta goes to size at matu
#Continue ERP of Meta, For simplicity with SJ and Psi in minimum vlaue
#This line is for low values of meta correct. For high values check with use of canonical equation if 
#Meta evolves or not. 
Meta=0
Xmax2 = NoMetaData[nrow(NoMetaData),1]-0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
Sj_init=Sj_min
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
FindErp <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(Meta,startpoint),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0,1), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
startpoint = c(0.5364282,
               4.697348,
               5.663510)
Meta = 0.0036985700
MetaESS4 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpoint,Meta),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)

colnames(MetaESS4$curvepoints)[2:4]<-Env
colnames(MetaESS4$curvepoints)[7:16]<-output
MetaESS4_data=as.data.frame(MetaESS4$curvepoints)
MetaESS4_data$R0_xx_yy = MetaESS4_data$`R0_xx[16]` - MetaESS4_data$`R0_yy[16]`
MetaESS4_data$ESS = ifelse(is.na(MetaESS4_data$R0_xx_yy), 'ESS', ifelse(MetaESS4_data$R0_xx_yy<0, "ERP", ifelse(MetaESS4_data$`R0_yy[16]`<0, "CSS", "EBP")))

ggplot(data=MetaESS1_data,aes(x=R2max,y=META))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))+
  geom_path(data=MetaESS4_data,aes(colour=ESS))

#######Strong trade-off######
S=2
Xmax2=0
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
BPfind<-PSPMequi(modelname="Stagestructured", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=Pars,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-Pars[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step 3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
NoMeta <- PSPMequi(modelname="Stagestructured",
                   biftype='EQ',
                   startpoint=c(Xmax2,startpnt),
                   stepsize=1, 
                   parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10), 
                   parameters=Pars,
                   options=c("popEVO", "0", 'parEVO',as.character(meta_index)),
                   clean=TRUE)
colnames(NoMeta$curvepoints)[2:4]<-Env
colnames(NoMeta$curvepoints)[5:(ncol(NoMeta$curvepoints)-3)]<-output
NoMetaData=as.data.frame(NoMeta$curvepoints)
NoMetaData=NoMetaData[c(1:108),]
#Step B: Use canonical equation of AD to find ESS where metamorphosis evolves to
Xmax2 = NoMetaData[nrow(NoMetaData),1]+0.5
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
meta_init=0
Sj_init=Sj_min
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
canonical_figD <- PSPMevodyn(modelname="Stagestructured", 
                             startpoint=c(startpoint, Psi,meta_init,Sj_init), 
                             curvepars=c(0.00000000001,10000000),
                             evopars=c(0,psi_index,Psi_min,1, 0, meta_index,0,1,0,Sj_index,Sj_init,MaxSj),
                             covars = NULL,
                             parameters = Pars,
                             options=NULL,
                             clean=TRUE)
colnames(canonical_figD$curvepoints)[2:4]<-Env
colnames(canonical_figD$curvepoints)[8:17]<-output
canonicalmeta_figD<-as.data.frame(canonical_figD$curvepoints)
#Continue with larger stepsize, to speed things up
Psi = max(Psi_min,canonicalmeta_figD$PSI[nrow(canonicalmeta_figD)])
Meta=canonicalmeta_figD$META[nrow(canonicalmeta_figD)]
Sj = max(Sj_min,canonicalmeta_figD$Xj[nrow(canonicalmeta_figD)])
startpnt=as.numeric(canonicalmeta_figD[nrow(canonicalmeta_figD), c(2:4)])
canonical_figDB <- PSPMevodyn(modelname="Stagestructured", 
                              startpoint=c(startpnt, Psi,Meta,Sj), 
                              curvepars=c(0.000001,1000000),
                              evopars=c(0,psi_index,Psi_min,1, Psi, meta_index,0,1,0,Sj_index,Sj_min,20),
                              covars = NULL,
                              parameters = Pars,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDB$curvepoints)[2:4]<-Env
colnames(canonical_figDB$curvepoints)[8:17]<-output
canonicalmeta_figDB<-as.data.frame(canonical_figDB$curvepoints)

#Bistability, find new equilibrium
Meta =  3.10053167E-02 
Sj=6.54583280E-03

Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
BPfind<-PSPMequi(modelname="Stagestructured", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=Pars,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-Pars[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
###Step 3: Continue non-triv equi over Xmax2 to find point where meta can evolve 
#Use metapar as parEVO, to see selectiongradient for this trait.
startpnt <- Nontriv$curvepoints[nrow(Nontriv$curvepoints),c(2:4)]
canonical_figDC <- PSPMevodyn(modelname="Stagestructured", 
                              startpoint=c(startpnt, Psi,Meta,Sj), 
                              curvepars=c(0.00001,1000000),
                              evopars=c(0,psi_index,Psi_min,1, Psi, meta_index,0,1,0,Sj_index,Sj_min,20),
                              covars = NULL,
                              parameters = Pars,
                              options=NULL,
                              clean=TRUE)
colnames(canonical_figDC$curvepoints)[2:4]<-Env
colnames(canonical_figDC$curvepoints)[8:17]<-output
canonicalmeta_figDC<-as.data.frame(canonical_figDC$curvepoints)





#Step C: ESS-continuation of Meta>0 equilibrium
#Step C1: Equilibrium with meta and Xj in ESS, Psi not since it is at its minimum. 
#Selection grad of psi_parameter is taken into account to check if it stays negative
Psi = max(Psi_min,canonicalmeta_figDC$PSI[nrow(canonicalmeta_figDC)])
Meta=canonicalmeta_figDC$META[nrow(canonicalmeta_figDC)]
Sj = max(Sj_min,canonicalmeta_figDC$Xj[nrow(canonicalmeta_figDC)])
startpnt=as.numeric(canonicalmeta_figDC[nrow(canonicalmeta_figDC), c(2:4)])
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)

MetaESS1 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta),
                     stepsize=0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)

MetaESS2 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpnt,Meta),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)

colnames(MetaESS1$curvepoints)[2:4]<-Env
colnames(MetaESS1$curvepoints)[7:16]<-output
MetaESS1_data=as.data.frame(MetaESS1$curvepoints)
MetaESS1_data$R0_xx_yy = MetaESS1_data$`R0_xx[16]` - MetaESS1_data$`R0_yy[16]`
MetaESS1_data$ESS = ifelse(is.na(MetaESS1_data$R0_xx_yy), 'ESS', ifelse(MetaESS1_data$R0_xx_yy<0, "ERP", ifelse(MetaESS1_data$`R0_yy[16]`<0, "CSS", "EBP")))

colnames(MetaESS2$curvepoints)[2:4]<-Env
colnames(MetaESS2$curvepoints)[7:16]<-output
MetaESS2_data=as.data.frame(MetaESS2$curvepoints)
MetaESS2_data$R0_xx_yy = MetaESS2_data$`R0_xx[16]` - MetaESS2_data$`R0_yy[16]`
MetaESS2_data$ESS = ifelse(is.na(MetaESS2_data$R0_xx_yy), 'ESS', ifelse(MetaESS2_data$R0_xx_yy<0, "ERP", ifelse(MetaESS2_data$`R0_yy[16]`<0, "CSS", "EBP")))

ggplot(data=MetaESS1_data,aes(x=R2max,y=META))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))

##Contintue with Meta = 1
Meta = 1
startpnt=as.numeric(MetaESS2_data[nrow(MetaESS2_data), c(1:4)])
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
MetaESS3<- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(startpnt),
                    stepsize=-0.1, 
                    parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 30), 
                    parameters=Pars,
                    options=c("popEVO", "0", 'parEVO',as.character(meta_index),
                              "popEVO", "0", 'parEVO',as.character(psi_index),
                              "popEVO", "0", 'parEVO',as.character(Sj_index)),
                    clean=TRUE)
#Check what happens after the limit point#
lp = MetaESS3$bifpoints
Xmax2=lp[1]-0.01

Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
BPfind<-PSPMequi(modelname="Stagestructured", 
                 biftype="EQ", 
                 startpoint=c(2,2,Xmax2), 
                 stepsize=-0.1,
                 parbnds=c(which(pars.names == 'Xmax1') - 1,0,5),
                 parameters=Pars,
                 options=c("popZE", "0"),
                 clean=TRUE,
                 force=FALSE)
colnames(BPfind$curvepoints)[2:(ncol(BPfind$curvepoints)-2)]<-c(Env,output)
colnames(BPfind$bifpoints)[2:(ncol(BPfind$bifpoints)-2)]<-c(Env,output)
bound.point <- as.numeric(BPfind$bifpoints[BPfind$biftypes == "BP #0"])[c(1:3)]
#Step2: Continue non-triv equi over xmax1 to find equilibrium values
Xmaxvalue<-Pars[which(pars.names == 'Xmax1')]
Nontriv <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(bound.point, 0),
                    stepsize=1, 
                    parbnds=c(which(pars.names == 'Xmax1') - 1, 0.0, Xmaxvalue), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
NontrivData=as.data.frame(Nontriv$curvepoints)
startpnt=as.numeric(NontrivData[nrow(NontrivData), c(2:4)])
canonical_Lp <- PSPMevodyn(modelname="Stagestructured", 
                           startpoint=c(startpnt, Psi,Meta,Sj), 
                           curvepars=c(0.00001,1000000),
                           evopars=c(0,psi_index,Psi_min,1, Psi, meta_index,0,1,0,Sj_index,Sj_min,MaxSj),
                           covars = NULL,
                           parameters = Pars,
                           options=NULL,
                           clean=TRUE)

#Meta stays 1, but size at meta goes to size at matu
#Continue ERP of Meta, For simplicity with SJ and Psi in minimum vlaue
#This line is for low values of meta correct. For high values check with use of canonical equation if 
#Meta evolves or not. 
Meta=0
Xmax2 = NoMetaData[nrow(NoMetaData),1]-0.1
startpoint = as.numeric(NoMetaData[nrow(NoMetaData),c(2:4)]) 
Sj_init=Sj_min
Pars=c(0.1,Xmax1,Xmax2,0.5,Q,0.1,Amax,Amin,1,Sig,Sb,Sj,Sm,Mu,xmin,
       Psi,Meta,Qs,Rho, S)
FindErp <- PSPMequi(modelname="Stagestructured",
                    biftype='EQ',
                    startpoint=c(Meta,startpoint),
                    stepsize=0.1, 
                    parbnds=c(which(pars.names == 'Meta') - 1, 0.0,1), 
                    parameters=Pars,
                    options=c("popEVO", "0"),
                    clean=TRUE)
startpoint = FindErp$bifpoints[c(2:4)]
Meta = FindErp$bifpoints[1]
MetaESS4 <- PSPMequi(modelname="Stagestructured",
                     biftype='ESS',
                     startpoint=c(Xmax2,startpoint,Meta),
                     stepsize=-0.1, 
                     parbnds=c(which(pars.names == 'Xmax2') - 1, 0.0, 10,0, meta_index,0,1), 
                     parameters=Pars,
                     options=c("popEVO", "0", 'parEVO',as.character(psi_index),
                               "popEVO", "0", 'parEVO',as.character(Sj_index)),
                     clean=TRUE)

colnames(MetaESS4$curvepoints)[2:4]<-Env
colnames(MetaESS4$curvepoints)[7:16]<-output
MetaESS4_data=as.data.frame(MetaESS4$curvepoints)
MetaESS4_data$R0_xx_yy = MetaESS4_data$`R0_xx[16]` - MetaESS4_data$`R0_yy[16]`
MetaESS4_data$ESS = ifelse(is.na(MetaESS4_data$R0_xx_yy), 'ESS', ifelse(MetaESS4_data$R0_xx_yy<0, "ERP", ifelse(MetaESS4_data$`R0_yy[16]`<0, "CSS", "EBP")))

ggplot(data=MetaESS1_data,aes(x=R2max,y=META))+geom_path(aes(colour=ESS))+
  geom_path(data=MetaESS2_data,aes(colour=ESS))+
  geom_path(data=MetaESS4_data,aes(colour=ESS))

######














